@@ -1,18 +1,69 @@
-# reactivity/reactive.py
+# pyright: reportMissingTypeStubs=false
 
 import types
-from typing import Sequence, Set, TypeVar
+from typing import (Any, Dict, ItemsView, Mapping, MutableMapping, MutableSequence, Optional, Sequence, Set, Tuple,
+                    TypeVar, Union, ValuesView, cast, overload)
 
 from reactivity.env import DEV
-from reactivity.flags import FLAG_OF_REACTIVE, FLAG_OF_SKIP, REACTIVITY_VALUE
+from reactivity.flags import FLAG_OF_REACTIVE, FLAG_OF_SKIP
+from reactivity.ref.definitions import Ref
 from reactivity.ref.utils import is_ref
 
+from .utils import (deep_to_raw, get_global_reactive_obj, is_in_global_original_object_map,
+                    is_in_global_reactive_object_map, is_marked_raw, is_reactive, mark_raw, reactive_class_map,
+                    reactive_reversed_class_map, record_new_reactive_obj, to_raw, track_reactive, track_reactive_value,
+                    trigger_reactive, trigger_reactive_value)
 from .vars import immutable_builtin_types, supported_builtin_types
-from .utils import (get_global_reactive_obj, is_in_global_original_object_map, is_in_global_reactive_object_map,
-                    is_marked_raw, is_reactive, mark_raw, reactive_class_map, reactive_reversed_class_map,
-                    record_new_reactive_obj, to_raw, track_reactive, trigger_reactive)
 
 T = TypeVar('T')
+U = TypeVar('U')
+
+
+@overload
+def _unref_and_reactive(obj: Ref[T]) -> T:
+    ...
+
+
+@overload
+def _unref_and_reactive(obj: T) -> T:
+    ...
+
+
+def _unref_and_reactive(obj: Union[Ref[T], T]) -> T:
+    return cast(Ref[T], obj).value if is_ref(obj) else reactive(cast(T, obj))
+
+
+class dict_items(ItemsView[T, U]):
+    _mapping: Mapping[T, U]
+
+    def __contains__(self, item: object) -> bool:
+        item = cast(Tuple[T, U], item)
+        key, value = item
+        try:
+            v = _unref_and_reactive(self._mapping[key])
+        except KeyError:
+            return False
+        else:
+            return v is value or v == value
+
+    def __iter__(self):
+        for key in self._mapping:
+            yield (key, _unref_and_reactive(self._mapping[key]))
+
+
+class dict_values(ValuesView[U]):
+    _mapping: Mapping[Any, U]
+
+    def __contains__(self, value: object) -> bool:
+        for key in self._mapping:
+            v = _unref_and_reactive(self._mapping[key])
+            if v is value or v == value:
+                return True
+        return False
+
+    def __iter__(self):
+        for key in self._mapping:
+            yield _unref_and_reactive(self._mapping[key])
 
 
 class ProxyMetaClass(type):
@@ -32,6 +83,8 @@ class ProxyMetaClass(type):
         # callable objects
         '__call__',
         # emulating container types
+        '__len__',
+        '__length_hint__',
         '__missing__',
         '__iter__',
         '__reversed__',
@@ -183,9 +236,9 @@ class ProxyMetaClass(type):
     dict_track_methods: Set[str] = {'copy', 'fromkeys', 'get', 'items', 'keys', 'reversed', 'values'}
     dict_trigger_methods: Set[str] = {'clear', 'pop', 'popitem', 'setdefault', 'update'}
 
-    def __new__(cls, name, bases, attrs):
+    def __new__(cls, name: str, bases: Tuple[type, ...], attrs: Dict[str, Any]) -> Optional[type]:
         # Check if None is in bases, if so, it means that this is the first call to __new__ and we should not do anything
-        if None in bases:
+        if type(None) in bases:
             return None
         proxy_cls = type.__new__(cls, name, bases, attrs)
 
@@ -224,6 +277,7 @@ class ProxyMetaClass(type):
                 cls.wrap_trigger_method(proxy_cls, set_trigger_method, patched_trigger_methods)
         # patch dict methods
         if issubclass(proxy_cls, dict):
+            cls.wrap_dict_view_method(proxy_cls, patched_track_methods)
             cls.wrap_dict_get_method(proxy_cls, patched_track_methods)
             for dict_track_method in cls.dict_track_methods:
                 cls.wrap_track_method(proxy_cls, dict_track_method, patched_track_methods)
@@ -244,7 +298,7 @@ class ProxyMetaClass(type):
         return proxy_cls
 
     @staticmethod
-    def wrap_attr_methods(proxy_cls, patched_track_methods: Set[str], patched_trigger_methods: Set[str]):
+    def wrap_attr_methods(proxy_cls: type, patched_track_methods: Set[str], patched_trigger_methods: Set[str]):
         # sourcery skip: assign-if-exp, reintroduce-else
 
         has_setattr = hasattr(proxy_cls, '__setattr__')
@@ -254,7 +308,7 @@ class ProxyMetaClass(type):
         if hasattr(proxy_cls, '__getattribute__'):
             raw__getattribute__ = getattr(proxy_cls, '__getattribute__')
 
-            def __getattribute__(self, name):
+            def __getattribute__(self: object, name: str):
                 # If the attribute is patched, return the original attribute and do not track it
                 is_magic: bool = name.startswith('__') and name.endswith('__')
                 if name in patched_track_methods or name in patched_trigger_methods or is_magic:
@@ -289,22 +343,46 @@ class ProxyMetaClass(type):
 
         if has_setattr:
 
-            def __setattr__(self, name, value):
+            def __setattr__(self: object, name: str, value: Any):
                 if is_reactive(value):
                     value = to_raw(value)
                 if is_in_global_original_object_map(self):
                     original = to_raw(self)
                     old_value = original.__getattribute__(name) if hasattr(self, name) else None
-                    if old_value == value:
-                        return
-                    original.__setattr__(name, value)
+                    if is_ref(old_value):
+                        old_value = cast(Ref[Any], old_value)
+                        if is_ref(value):
+                            value = cast(Ref[Any], value)
+                            if old_value.value == value.value:
+                                return
+                            original.__setattr__(name, value)
+                        else:
+                            if old_value.value == value:
+                                return
+                            old_value.value = value
+                    else:
+                        if old_value == value:
+                            return
+                        original.__setattr__(name, value)
                 else:
                     # 如果是一个自定义类，那在初始化响应式对象时，可能会执行一些操作，
                     # 但此时还没有将响应式对象和原始对象关联在一起，只对响应式对象操作即可
                     old_value = raw__getattribute__(self, name) if hasattr(self, name) else None
-                    if old_value == value:
-                        return
-                    raw__setattr__(self, name, value)
+                    if is_ref(old_value):
+                        old_value = cast(Ref[Any], old_value)
+                        if is_ref(value):
+                            value = cast(Ref[Any], value)
+                            if old_value.value == value.value:
+                                return
+                            raw__setattr__(self, name, value)
+                        else:
+                            if old_value.value == value:
+                                return
+                            old_value.value = value
+                    else:
+                        if old_value == value:
+                            return
+                        raw__setattr__(self, name, value)
                 trigger_reactive(self, name)
                 if DEV:
                     print(
@@ -317,7 +395,7 @@ class ProxyMetaClass(type):
         if hasattr(proxy_cls, '__delattr__'):
             raw__delattr__ = getattr(proxy_cls, '__delattr__')
 
-            def __delattr__(self, name):
+            def __delattr__(self: object, name: str):
                 if is_in_global_original_object_map(self):
                     original = to_raw(self)
                     original.__delattr__(name)
@@ -333,7 +411,7 @@ class ProxyMetaClass(type):
         if hasattr(proxy_cls, '__dir__'):
             raw__dir__ = getattr(proxy_cls, '__dir__')
 
-            def __dir__(self):
+            def __dir__(self: object):
                 track_reactive(self, '__dir__')
                 if DEV:
                     print(f'''[Reactive] __dir__: self={repr(self)} at {hex(id(self))} ({id(self)})''')
@@ -348,50 +426,27 @@ class ProxyMetaClass(type):
             patched_track_methods.add('__dir__')
 
     @staticmethod
-    def wrap_dict_get_method(proxy_cls, patched_track_methods: Set[str]):
-        has_setitem = hasattr(proxy_cls, '__setitem__')
-
-        if has_setitem:
-            raw__setitem__ = getattr(proxy_cls, '__setitem__')
-
-        if hasattr(proxy_cls, 'get'):
-            raw_get = getattr(proxy_cls, 'get')
-
-            def get(self, key, default=None):
-                original = to_raw(self)
-                track_reactive(self, REACTIVITY_VALUE)
-                if DEV:
-                    print(
-                        f'''[Reactive] track(get): key={key}, default={default}, self={repr(self)} at {hex(id(self))} ({id(self)})'''
-                    )
-                result = original.get(key, default)
-                return result.value if is_ref(result) else reactive(result)
-
-            setattr(proxy_cls, 'get', get)
-            patched_track_methods.add('get')
-
-    @staticmethod
-    def wrap_item_methods(proxy_cls, patched_methods: Set[str], patched_trigger_methods: Set[str]):
+    def wrap_item_methods(proxy_cls: type, patched_methods: Set[str], patched_trigger_methods: Set[str]):
         # sourcery skip: assign-if-exp, reintroduce-else
         has_getitem = hasattr(proxy_cls, '__getitem__')
         has_setitem = hasattr(proxy_cls, '__setitem__')
 
-        if has_setitem:
-            raw__setitem__ = getattr(proxy_cls, '__setitem__')
+        # if has_setitem:
+        # raw__setitem__ = getattr(proxy_cls, '__setitem__')
 
         if has_getitem:
-            raw__getitem__ = getattr(proxy_cls, '__getitem__')
+            # raw__getitem__ = getattr(proxy_cls, '__getitem__')
 
-            def __getitem__(self, key):
+            def __getitem__(self: Union[Sequence[Any], Mapping[Any, Any]], key: slice):
                 original = to_raw(self)
-                track_reactive(self, REACTIVITY_VALUE)
+                track_reactive_value(self)
                 if DEV:
                     print(
                         f'''[Reactive] track(__getitem__): key={key}, self={repr(self)} at {hex(id(self))} ({id(self)})'''
                     )
                 result = original.__getitem__(key)
                 if is_ref(result) and not isinstance(self, list):
-                    return result.value
+                    return cast(Ref[Any], result).value
                 return reactive(result)
 
             setattr(proxy_cls, '__getitem__', __getitem__)
@@ -399,17 +454,29 @@ class ProxyMetaClass(type):
 
         if has_setitem:
 
-            def __setitem__(self, key, value):
+            def __setitem__(self: Union[MutableSequence[Any], MutableMapping[Any, Any]], key: slice, value: Any):
                 if is_reactive(value):
                     value = to_raw(value)
                 original = to_raw(self)
                 # old_value = raw__getitem__(self, key) if key in self else None
                 old_value = original.__getitem__(key) if key in self else None
-                if old_value == value:
-                    return
-                # raw__setitem__(self, key, value)
-                original.__setitem__(key, value)
-                trigger_reactive(self, REACTIVITY_VALUE)
+                if is_ref(old_value):
+                    old_value = cast(Ref[Any], old_value)
+                    if is_ref(value):
+                        value = cast(Ref[Any], value)
+                        if old_value.value == value.value:
+                            return
+                        original.__setitem__(key, value)
+                    else:
+                        if old_value.value == value:
+                            return
+                        old_value.value = value
+                else:
+                    if old_value == value:
+                        return
+                    # raw__setitem__(self, key, value)
+                    original.__setitem__(key, value)
+                trigger_reactive_value(self)
                 if DEV:
                     print(
                         f'''[Reactive] trigger(__setitem__): key={key}, value={value}, self={repr(self)} at {hex(id(self))} ({id(self)})'''
@@ -419,15 +486,15 @@ class ProxyMetaClass(type):
             patched_trigger_methods.add('__setitem__')
 
     @staticmethod
-    def wrap_track_method(proxy_cls, method_name: str, patched_methods: Set[str]) -> None:
+    def wrap_track_method(proxy_cls: type, method_name: str, patched_methods: Set[str]) -> None:
         if not hasattr(proxy_cls, method_name) or method_name in patched_methods:
             return
 
-        raw_method = getattr(proxy_cls, method_name)
+        # raw_method = getattr(proxy_cls, method_name)
 
-        def wrapper(self, *args, **kwargs):
+        def wrapper(self: object, *args: Any, **kwargs: Any):
             original = to_raw(self)
-            track_reactive(self, REACTIVITY_VALUE)
+            track_reactive_value(self)
             if DEV:
                 print(
                     f'''[Reactive] track({method_name}): args={args}, kwargs={kwargs}, self={repr(self)} at {hex(id(self))} ({id(self)})'''
@@ -440,17 +507,17 @@ class ProxyMetaClass(type):
         patched_methods.add(method_name)
 
     @staticmethod
-    def wrap_trigger_method(proxy_cls, method_name: str, patched_methods: Set[str]) -> None:
+    def wrap_trigger_method(proxy_cls: type, method_name: str, patched_methods: Set[str]) -> None:
         if not hasattr(proxy_cls, method_name) or method_name in patched_methods:
             return
 
-        raw_method = getattr(proxy_cls, method_name)
+        # raw_method = getattr(proxy_cls, method_name)
 
-        def wrapper(self, *args, **kwargs):
+        def wrapper(self: object, *args: Any, **kwargs: Any):
             original = to_raw(self)
             # result = raw_method(self, *args, **kwargs)
             result = original.__getattribute__(method_name)(*args, **kwargs)
-            trigger_reactive(self, REACTIVITY_VALUE)
+            trigger_reactive_value(self)
             if DEV:
                 print(
                     f'''[Reactive] trigger({method_name}): args={args}, kwargs={kwargs}, self={repr(self)} at {hex(id(self))} ({id(self)})'''
@@ -460,6 +527,59 @@ class ProxyMetaClass(type):
         setattr(proxy_cls, method_name, wrapper)
         patched_methods.add(method_name)
 
+    @staticmethod
+    def wrap_dict_get_method(proxy_cls: type, patched_track_methods: Set[str]):
+        if hasattr(proxy_cls, 'get'):  # Fool-proofing
+
+            def get(self: Dict[Any, Any], key: Any, default: Any = None):
+                original = to_raw(self)
+                track_reactive_value(self)
+                if DEV:
+                    print(
+                        f'''[Reactive] track(get): key={key}, default={default}, self={repr(self)} at {hex(id(self))} ({id(self)})'''
+                    )
+                result = original.get(key, default)
+                return result.value if is_ref(result) else reactive(result)
+
+            setattr(proxy_cls, 'get', get)
+            patched_track_methods.add('get')
+
+    @staticmethod
+    def wrap_dict_view_method(proxy_cls: type, patched_track_methods: Set[str]):
+        if hasattr(proxy_cls, 'items'):  # Fool-proofing
+
+            def items(self: Dict[T, U]) -> dict_items[T, U]:
+                original: Dict[T, U] = to_raw(self)
+                track_reactive_value(self)
+                if DEV:
+                    print(f'''[Reactive] track(items): self={repr(self)} at {hex(id(self))} ({id(self)})''')
+                return dict_items(original)
+
+            setattr(proxy_cls, 'items', items)
+            patched_track_methods.add('items')
+
+        if hasattr(proxy_cls, 'values'):
+
+            def values(self: Dict[Any, U]) -> dict_values[U]:
+                original: Dict[Any, U] = to_raw(self)
+                track_reactive_value(self)
+                if DEV:
+                    print(f'''[Reactive] track(values): self={repr(self)} at {hex(id(self))} ({id(self)})''')
+                return dict_values(original)
+
+            setattr(proxy_cls, 'values', values)
+            patched_track_methods.add('values')
+
+
+@overload
+def reactive(instance: Dict[U, Any]) -> Dict[U, Any]:
+    ...
+
+
+@overload
+def reactive(instance: T) -> T:
+    ...
+
 
 def reactive(instance: T) -> T:
 
@@ -507,7 +627,7 @@ def reactive(instance: T) -> T:
     return __create_proxy(instance, init_with_instance=False)
 
 
-def get_patched_class(instance):
+def get_patched_class(instance: object):
     raw_class = instance.__class__
     if raw_class not in reactive_class_map:
         class_name = raw_class.__name__
@@ -516,7 +636,7 @@ def get_patched_class(instance):
     return reactive_class_map[raw_class]
 
 
-def __create_proxy(instance, init_with_instance=True):
+def __create_proxy(instance: object, init_with_instance: bool = True):
     if DEV:
         print(f'[Reactive] create proxy: {instance}')
     patched_class = get_patched_class(instance)
@@ -525,4 +645,4 @@ def __create_proxy(instance, init_with_instance=True):
     return record_new_reactive_obj(instance, patched_class())
 
 
-__all__ = ['reactive', 'to_raw', 'mark_raw']
+__all__ = ['is_reactive', 'mark_raw', 'reactive', 'to_raw', 'deep_to_raw']
